@@ -3,10 +3,8 @@ from paramiko import SSHClient,AutoAddPolicy
 from io import StringIO
 import paramiko
 import os, sys, traceback, time, re, random, glob
-import logging
+import logging, stat
 import hashlib
-import queue, threading
-import subprocess, shlex, socket
 import __main__
 try:
     from sshscriptdollar import SSHScriptDollar
@@ -14,10 +12,7 @@ try:
 except ImportError:
     from .sshscriptdollar import SSHScriptDollar
     from .sshscripterror import SSHScriptError, SSHScriptExit
-"""
-# replace @{var} in py scripts
-pvarS = re.compile('(\b?)\@\{(.+?)\}',re.S)
-"""
+
 # replace @{var} in $shell-command
 pvar = re.compile('(\b?)\@\{(.+)\}')
 # replace $.stdout, $.stderr to _c.stdout, _c.stderr, $.host in $LINE
@@ -32,7 +27,7 @@ pDollarWithoutDot = re.compile('^\$[^\.][\{\w]?')
 # but not ${, $${
 pDollarCommand = re.compile('^\$(?!\$?\{)') # will match 
 # replace @open( to self.open( in Py
-pAtFuncS = re.compile('^([\t ]*?)\@(\w+\()',re.M)
+#pAtFuncS = re.compile('^([\t ]*?)\@(\w+\()',re.M)
 # find $.stdin =
 pStdinEquals = re.compile('^[\t ]*?(\$\.stdin[\t ]*=)',re.M)
 # find "with $shell-command"
@@ -50,6 +45,7 @@ pCurlyBrackets = re.compile('^[ \t]*?\}')
 pqoute1 =re.compile('("{3})(.+?)"{3}',re.S)
 pqoute2 =re.compile("('{3})(.+?)'{3}",re.S)
 
+"""
 class DummyLogger:
     def __init__(self,level=2) -> None:
         # 1:debug, 2:info, 3:warning, 4:error
@@ -62,8 +58,11 @@ class DummyLogger:
         if self.level <= 3 : print(f'[WARNING]{msg}')        
     def error(self,msg):
         if self.level <= 4 : print(f'[ERROR]{msg}')        
-
 logger = DummyLogger()
+"""
+
+global sshscriptLogger
+sshscriptLogger = None
 
 class LineGenerator:
     def __init__(self,lines):
@@ -88,12 +87,12 @@ class LineGenerator:
 # replace $.stdout, $.stderr to _c.stdout, _c.stderr
 # replace $.open(), $.subopen() to SSHScript.inContext.
 pQuoted = re.compile('\(.*\)')
-# exported to $.<func>, such as $.open, $.close, $.exit
-SSHScriptExportedFuncs = set()
+# exported to $.<func>, such as $.open, $.close, $.exit, $.sftp,
+SSHScriptExportedNames = set(['sftp','client']) #
 #SSHScriptClsExportedFuncs = set() # "include" only, now.
 def pstdSub(m):
     pre,post = m.groups()
-    if post in SSHScriptExportedFuncs:
+    if post in SSHScriptExportedNames:
         return f'{pre}SSHScript.inContext.{post}'
     elif post in SSHScriptDollar.exportedProperties:
         return f'{pre}_c.{post}'
@@ -105,7 +104,7 @@ def pstdSub(m):
 
 def export2Dollar(func):
     try:
-        SSHScriptExportedFuncs.add(func.__name__)
+        SSHScriptExportedNames.add(func.__name__)
     except AttributeError:
         # classmethod
         #assert isinstance(func,classmethod)
@@ -114,25 +113,41 @@ def export2Dollar(func):
     return func
 
 class SSHScript(object):
+    logger = sshscriptLogger
     # this SSHScript() which in context of @method in py
     inContext = None
     # a lookup table of instances of SSHScript by id
     items = {}
 
     @classmethod
-    def connect(cls,host,port=22,username=None,password=None,**kw):
+    def connectClient(cls,host,username=None,password=None,port=22,**kw):
         client = SSHClient()
         #client.load_system_host_keys(os.path.expanduser('~/.ssh/known_hosts'))
         # 允許連線不在know_hosts檔案中的主機
         client.set_missing_host_key_policy(AutoAddPolicy())
         #3.連線伺服器
-        client.connect(host,port=port,username=username,password=password,**kw)
+        client.connect(host,username=username,password=password,port=port,**kw)
         return client
     
     @classmethod
-    def include(cls,prefix,abspath):
+    def include(cls,prefix,abspath,alreadyIncluded=None):
+        if alreadyIncluded is None:
+            alreadyIncluded = {}
+        
         if not os.path.exists(abspath):
             raise SSHScriptError(f'{abspath} not fournd, @include("{abspath}") failed',401)
+        
+        # prevent infinite loop
+        try:
+            alreadyIncluded[abspath] += 1
+            maxInclude = int(os.environ.get('MAX_INCLUDE',100))
+            if alreadyIncluded[abspath] > maxInclude:
+                raise SSHScriptError(f'{abspath} have been included over {maxInclude} times, guessing this is infinite loop',404)
+        except KeyError:
+            alreadyIncluded[abspath] = 1
+        
+
+
         with open(abspath) as fd:
             content = fd.read()
         #find prefix
@@ -156,7 +171,7 @@ class SSHScript(object):
             def pAtIncludeSub(m):
                 prefix, path = m.groups()
                 abspath = os.path.join(os.path.dirname(scriptPath),eval(path))
-                content = SSHScript.include(prefix,abspath)
+                content = SSHScript.include(prefix,abspath,alreadyIncluded)
                 return content
             script = pAtInclude.sub(pAtIncludeSub,script)
 
@@ -224,7 +239,7 @@ class SSHScript(object):
         raise SSHScriptExit(code)
     
     @export2Dollar
-    def open(self,host,username=None,password=None,port=22,**kw):
+    def connect(self,host,username=None,password=None,port=22,**kw):
         # host 可以是 username@hostname 的格式
         if '@' in host:
             username,host = host.split('@')
@@ -235,7 +250,7 @@ class SSHScript(object):
             if host == self.host:
                 raise SSHScriptError(f'Self connection from {host} to {host} is not allowed',502)
             else:
-                return self.subopen(host,username,password,port,**kw)
+                return self.subconnect(host,username,password,port,**kw)
         
         # self.host　是用來判斷有沒有ssh連線的依據（如果沒有則是subprocess)
         self.host = host
@@ -249,22 +264,22 @@ class SSHScript(object):
             if 'proxyCommand' in kw:
                 raise NotImplementedError('proxyCommand not support in nested session')
             else:
-                logger.debug(f'{self.id} nested openning {self.username}@{self.host}:{self.port}')
+                sshscriptLogger.debug(f'{self.id} nested openning {self.username}@{self.host}:{self.port}')
                 # REF: https://stackoverflow.com/questions/35304525/nested-ssh-using-python-paramiko
                 vmtransport = self.parentSession._client.get_transport()
                 dest_addr = (host,port)
                 local_addr = (self.parentSession.host,self.parentSession.port)
                 self._sock = vmtransport.open_channel("direct-tcpip", dest_addr, local_addr)
-                self._client = SSHScript.connect(host,port,username,password,sock=self._sock,**kw)
+                self._client = SSHScript.connectClient(host,username,password,port,sock=self._sock,**kw)
         else:
             if 'proxyCommand' in kw:
-                logger.debug(f'{self.id} openning {self.username}@{self.host}:{self.port} by {kw["proxyCommand"]}')
+                sshscriptLogger.debug(f'{self.id} openning {self.username}@{self.host}:{self.port} by {kw["proxyCommand"]}')
                 self._sock = self.getSocketWithProxyCommand(kw['proxyCommand'])
                 del kw['proxyCommand']
-                self._client = SSHScript.connect(host,port,username,password,sock=self._sock,**kw)
+                self._client = SSHScript.connectClient(host,username,password,port,sock=self._sock,**kw)
             else:
-                logger.debug(f'{self.id} openning {self.username}@{self.host}:{self.port}')
-                self._client = SSHScript.connect(host,port,username,password,**kw)
+                sshscriptLogger.debug(f'{self.id} openning {self.username}@{self.host}:{self.port}')
+                self._client = SSHScript.connectClient(host,username,password,port,**kw)
 
         self._previousSshscriptInContext = SSHScript.inContext
         SSHScript.inContext = self
@@ -272,11 +287,15 @@ class SSHScript(object):
         return self
     
     @export2Dollar
-    def subopen(self,host,username=None,password=None,port=22,**kw):
+    def open(self,host,username=None,password=None,port=22,**kw):
+        return self.connect(host,username,password,port,**kw)
+    
+    @export2Dollar
+    def subconnect(self,host,username=None,password=None,port=22,**kw):
         # enter a new channel
         if '@' in host: username,host = host.split('@')
         nestedSession = SSHScript(self)
-        nestedSession.open(host,username,password,port,**kw)
+        nestedSession.connect(host,username,password,port,**kw)
         return nestedSession
     
     @export2Dollar
@@ -302,7 +321,7 @@ class SSHScript(object):
             self._sock = None
         
         if self._client:
-            logger.debug(f'{self.id} closing {self.username}@{self.host}:{self.port}')
+            #sshscriptLogger.debug(f'{self.id} closing {self.username}@{self.host}:{self.port}')
             self._client.close()
             self._client = None
         
@@ -323,15 +342,19 @@ class SSHScript(object):
         self.username = None
 
         SSHScript.inContext = self._previousSshscriptInContext
-        logger.debug(f'restore  SSHScript.inContext to {SSHScript.inContext}')        
 
     def __del__(self):
         self.close(True)
-        del SSHScript.items[self.id]
+        try:
+            del SSHScript.items[self.id]
+        except KeyError:
+            # when calls $.exit in runScript() would cause this error
+            pass
+        
 
     #def getRsaKeyAtPath(self,pathOfRsaPrivate):
     @export2Dollar
-    def getkey(self,pathOfRsaPrivate):
+    def pkey(self,pathOfRsaPrivate):
         #pathOfRsaPrivate = f'/home/{$.username}/.ssh/id_rsa'
         if self.host:
             _,stdout,_ = self._client.exec_command(f'cat "{pathOfRsaPrivate}"')
@@ -341,10 +364,9 @@ class SSHScript(object):
             with open(pathOfRsaPrivate) as fd:
                 return paramiko.RSAKey.from_private_key(fd)
     
-    
-    @export2Dollar
-    def timeout(self,v):
-        self._timeout = float(v)
+    #@export2Dollar
+    #def timeout(self,v):
+    #    self._timeout = float(v)
     
 
     @property
@@ -361,17 +383,29 @@ class SSHScript(object):
         if self.subSession:
             return self.subSession.upload(src,dst)
 
-        if not src.startswith('/'):
-            raise SSHScriptError(f'uploading src "{src}" must be absolute path',503)
+        #if not src.startswith('/'):
+        #    raise SSHScriptError(f'uploading src "{src}" must be absolute path',503)
+        src = os.path.abspath(src)
+        if not os.path.exists(src):
+            raise FileNotFoundError(src)
+        if not os.path.isfile(src):
+            raise SSHScriptError(f'uploading src "{src}" must be a file',503)
         
         if not dst.startswith('/'):
             raise SSHScriptError(f'uploading dst "{dst}" must be absolute path',504)
         
         
-        logger.debug(f'upload {src} to {dst}')
+        sshscriptLogger.debug(f'upload {src} to {dst}')
 
         # check exists of dst folders
+        srcbasename = os.path.basename(src)
+        dstbasename = os.path.basename(dst)
         if makedirs:
+            # 在此情形下，假設使用者給的是目錄，而非檔案名稱
+            # 如果需要產生目錄，需要全部給的目錄都產生
+            if not dstbasename == srcbasename:
+                dst = os.path.join(dst,srcbasename)
+
             def checking(dst,foldersToMake):
                 dstDir = os.path.dirname(dst)
                 try:
@@ -380,19 +414,43 @@ class SSHScript(object):
                     foldersToMake.append(dstDir)
                     return checking(dstDir,foldersToMake)
                 return foldersToMake
+            # 由下層往上層檢查哪些目錄不存在（假設最後一層是檔案名稱）
             foldersToMake = checking(dst,[])
             if len(foldersToMake):
                 foldersToMake.reverse()
                 for folder in foldersToMake:
                     self.sftp.mkdir(folder)
-                    logger.debug(f'mkdir {folder}')
-        
-        if not overwrite:
-            try:
-                self.sftp.stat(dst)
-                raise SSHScriptError(f'{dst} already existed',403)
-            except FileNotFoundError:
+                    sshscriptLogger.debug(f'mkdir {folder}')
+        else:
+            # 1. basename 一樣：
+            # 2. basename 不一樣：
+            #    2.1 dst 為目錄： dst+= basename
+            #    2.2 dst 為檔案：
+            if dstbasename == srcbasename:
                 pass
+            else:
+                try:
+                    dststat = self.sftp.stat(dst)
+                except FileNotFoundError:
+                    # 不是，dst 視為檔案
+                    pass
+                else:
+                    # https://stackoverflow.com/questions/18205731/how-to-check-a-remote-path-is-a-file-or-a-directory
+                    if stat.S_ISDIR(dststat.st_mode):
+                        # is folder
+                        dst = os.path.join(dst,srcbasename)
+                        try:
+                            dststat = self.sftp.stat(dst)
+                        except FileNotFoundError:
+                            pass
+                        else:
+                            if not overwrite:
+                                raise FileExistsError(f'{dst} already exists')
+                    else:
+                        # is file
+                        if not overwrite:
+                            raise FileExistsError(f'{dst} already exists')
+        
         self.sftp.put(src,dst)
 
     @export2Dollar
@@ -405,9 +463,14 @@ class SSHScript(object):
             raise SSHScriptError(f'downloading src "{src}" must be absolute path',505)
         
         if not dst.startswith('/'):
-            raise SSHScriptError(f'downloading dst "{dst}" must be absolute path',506)
+            #raise SSHScriptError(f'downloading dst "{dst}" must be absolute path',506)
+            dst = os.path.abspath(dst)
         
-        logger.debug(f'downaload from {src} to {dst}')            
+        # if dst is a folder, append the filename of src to dst
+        if os.path.isdir(dst):
+            dst = os.path.join(dst,os.path.basename(src))
+
+        sshscriptLogger.debug(f'downaload from {src} to {dst}')            
         
         self.sftp.get(src,dst)
 
@@ -436,7 +499,7 @@ class SSHScript(object):
         # replace with @open( to "with open("
         script = pAtFuncSWith.sub('\\1with SSHScript.inContext.\\3',script)
         # replace @open( to open (example)
-        script = pAtFuncS.sub('\\1SSHScript.inContext.\\2',script)
+        #script = pAtFuncS.sub('\\1SSHScript.inContext.\\2',script)
         listOfLines = script.split('\n')
         # listOfLines: [string]
         lines = LineGenerator(listOfLines)
@@ -668,7 +731,7 @@ class SSHScript(object):
         def dumpScriptItem(scriptChunk):
             for idx, line in enumerate(scriptChunk.split('\n')):
                 print(f'{str(idx+1).zfill(3)}:{line}')
-            traceback.print_exc()
+            #traceback.print_exc()
 
         for scriptChunk in self.blocksOfScript:
             if isinstance(scriptChunk, str):
@@ -688,15 +751,14 @@ class SSHScript(object):
                     if os.environ.get('DEBUG'):  dumpScriptItem(scriptChunk)
                     raise
                 except SystemExit as e:
-                    if e.code != 0:
+                    if e.code:
                         traceback.print_exc()
-                    else:
-                        raise
+                    raise
                 except SSHScriptError:
                     if os.environ.get('DEBUG'):  dumpScriptItem(scriptChunk)
                     raise
                 except SSHScriptExit:
-                    break
+                    raise
                 except:
                     if os.environ.get('DEBUG'):  dumpScriptItem(scriptChunk)
                     raise
@@ -704,39 +766,52 @@ class SSHScript(object):
                 scriptChunk.run(None,_locals,_globals,showScript=showScript)
         return _locals
 
-def runPath(givenPaths,varGlobals=None,varLocals=None,showScript=False,showFilesOrder=False,unisession=True):
-    # @givenPaths:string 
-    # @showScript:bool
-    # @varGlobals:dict
-    # @unisession:bool, if true, use the same session(an instance of SSHScript) for all files.
+def runFile(givenPaths,
+        varGlobals=None,
+        varLocals=None,
+        showScript=False,
+        showRunOrder=False,
+        unisession=True):
+    """
+    @unisession:bool, if true, use the same session(an instance of SSHScript) 
+                for all files.
+    """
+    
+    if isinstance(givenPaths, str): givenPaths = [givenPaths]
+    
+    ext = os.environ.get('SSHSCRIPT_EXT','.spy')
+
     paths = []
     for path in givenPaths:
-        path = os.path.abspath(path)
+        abspath = os.path.abspath(path)
         # if path is a directory, add all *.spy files in it
-        if os.path.isdir(path):
+        if os.path.isdir(abspath):
             # files in folder are sorted by name
-            unsortedFilesInPath = list(filter(lambda x: x[-4:] == '.spy',[os.path.abspath(os.path.join(path,y)) for y in os.listdir(path)]))
+            unsortedFilesInPath = list(filter(lambda x: x[-4:] == ext,[os.path.abspath(os.path.join(path,y)) for y in os.listdir(path)]))
         else:
             # glob.glob returns a list of files, or empty list if no file matches
-            unsortedFilesInPath = glob.glob(path)
+            unsortedFilesInPath = list(filter(lambda x: os.path.splitext(x)[1] == ext ,glob.glob(abspath)))
             if len(unsortedFilesInPath) == 0:
-                if not os.path.exists(path):
-                    # Don't raise exception, just ignore this path
-                    # Because in context of sshscript, this may be a argument to the script
-                    #raise RuntimeError(f'{os.path.abspath(path)} not found')
-                    pass
-                elif os.path.isfile(path):
-                    unsortedFilesInPath.append(path)
+                if not os.path.exists(abspath):
+                    # In context of sshscript, this may be a argument to the script.
+                    # If the script also wants to accept command line arguments,
+                    # it should assign the argument in form of
+                    # --arg=value , not in form of --arg value
+                    # otherwise, this exception would raised
+                    raise RuntimeError(f'{os.path.abspath(path)} not found')
+                elif os.path.isfile(abspath):
+                    unsortedFilesInPath.append(abspath)
                 else:
-                    raise RuntimeError(f'{os.path.abspath(path)} not supported')
+                    raise RuntimeError(f'{abspath} not supported')
+        
         unsortedFilesInPath.sort()
         for p in unsortedFilesInPath:
             if p in paths:
-                logger.debug('ignore duplicate path: %s' % p)
+                sshscriptLogger.debug('ignore duplicate path: %s' % p)
                 continue
             paths.append(p)
 
-    if showFilesOrder:
+    if showRunOrder:
         for path in paths:
             print(path)
         return
@@ -753,21 +828,33 @@ def runPath(givenPaths,varGlobals=None,varLocals=None,showScript=False,showFiles
         # 每一個檔案產生一個sshscript物件
         if not unisession:
             session = SSHScript()
-        logger.debug(f'run {file}')
+
+        sshscriptLogger.debug(f'run {file}')
+
         absfile = os.path.abspath(file)
         with open(absfile) as fd:
             script = fd.read()
 
-        _locals['__file__'] = absfile
-        newlocals = session.run(script,_locals.copy(),_globals.copy(),showScript=showScript)
-        if newlocals.get('__export__'):
-            # __export__ = ['*'] will export all
-            if '*' in newlocals['__export__']:
-                _locals.update(newlocals)
+        try:
+            _locals['__file__'] = absfile
+            newglobals = session.run(script,_locals.copy(),_globals.copy(),showScript=showScript)
+        except SSHScriptExit:
+            sshscriptLogger.debug('exit by receiving SSHScriptExit')
+            break
+        
+        exported = newglobals.get('__export__')        
+        if exported:
+            # __export__ = '*' will export all
+            if '*' == exported:
+                #_locals.update(newlocals)
+                _globals.update(newglobals)
             else:
-                for key in newlocals['__export__']:
-                    logger.debug(f'{absfile} export {key}')
-                    _locals[key] = newlocals[key]
+                basename = os.path.basename(file)
+                for key in exported:
+                    sshscriptLogger.debug(f'{basename} export {key}')
+                    #_locals[key] = newlocals[key]
+                    _globals[key] = newglobals[key]
+        
         if not unisession:    
             session.close()
             del session
@@ -775,7 +862,7 @@ def runPath(givenPaths,varGlobals=None,varLocals=None,showScript=False,showFiles
     if unisession:    
         session.close()
         del session
-
+"""
 def runScript(script,varGlobals=None,varLocals=None,showScript=False):
     session = SSHScript()
     _locals = locals()
@@ -783,32 +870,52 @@ def runScript(script,varGlobals=None,varLocals=None,showScript=False):
     if varGlobals: _globals.update(varGlobals)
     if varLocals: _locals.update(varLocals)
     session.run(script,varGlobals=_globals,varLocals=_locals,showScript=showScript)
+"""
+
+def runScript(script,varGlobals=None,varLocals=None):
+    
+    session = SSHScript()
+    session.run(script,varGlobals,varLocals,showScript=False)
+
+def setupLogger():
+    global sshscriptLogger
 
-def setupLogger(debug=None):
-    global logger
     # silent paramiko
-    logging.getLogger("paramiko").setLevel(logging.WARNING)
+    #logging.getLogger("paramiko").setLevel(logging.WARNING)
 
-    logger = logging.getLogger('SSHScript')
-    if debug:
-        logger.setLevel(logging.DEBUG) 
+    sshscriptLogger = logging.getLogger('sshscript')
+    SSHScript.logger = sshscriptLogger    
+
+    if os.environ.get('DEBUG'):
+        sshscriptLogger.setLevel(logging.DEBUG) 
     else:
-        logger.setLevel(logging.INFO) 
-    handler = logging.StreamHandler(sys.stdout)
-    #handler.setLevel(logging.DEBUG)    
-    #handler = logging.FileHandler('test.log', 'w', 'utf-8') # or whatever
-    #handler.setFormatter(logging.Formatter('%(name)s %(message)s')) # or whatever
-    logger.addHandler(handler)
+        sshscriptLogger.setLevel(logging.INFO) 
+    
+    if os.environ.get('SILENT'):
+        pass
+    elif sys.stdout.isatty() or os.environ.get('VERBOSE'):
+        handler = logging.StreamHandler(sys.stdout)
+        handler.setFormatter(logging.Formatter('%(asctime)s:: %(message)s',"%m-%d %H:%M:%S")) # or whatever
+        sshscriptLogger.addHandler(handler)
 
 
-def main(topic,args):
-    if logger is None:
-        setupLogger(args.debug)
-    if topic == '-f':
-        runPath(args.paths,showScript=args.showScript,showFilesOrder=args.showFilesOrder)
+def main(args):
+   
+    if args.debug:
+        os.environ['DEBUG'] = '1'
+
+    if args.silent:
+        os.environ['SILENT'] = '1'
+    elif args.verbose:
+        os.environ['VERBOSE'] = '1'
+    
+    runFile(args.paths,
+        varGlobals=None,
+        varLocals=None,
+        showScript=args.showScript,
+        showRunOrder=args.showRunOrder,
+        unisession=True)
 
-__main__.SSHScript = SSHScript
-__main__.logger = logger
 
 def run():
     import argparse
@@ -816,31 +923,57 @@ def run():
     # REF: https://stackoverflow.com/questions/15753701/how-can-i-pass-a-list-as-a-command-line-argument-with-argparse
     parser = argparse.ArgumentParser(description='SSHScript')
 
-    parser.add_argument('--script', dest='showScript', action='store_const',
-                        const='1', default='',
-                        help='show the converted python script only, no execution.')
-    
-    parser.add_argument('--file-order', dest='showFilesOrder', action='store_const',
-                        const='1', default='',
+    parser.add_argument('--run-order', dest='showRunOrder', action='store_true',
+                        default=False,
                         help='show the files to run in order, no execution.')
 
+    parser.add_argument('--script', dest='showScript', action='store_true',
+                        default=False,
+                        help='show the converted python script only, no execution.')
+    
+    parser.add_argument('--silent', dest='silent', action='store_true',
+                        default=False,
+                        help='when executing on tty, do not dump to console.')
 
-    # set log level to debug
-    parser.add_argument('--debug', dest='debug', action='store_const',
-                        const='1', default='',
+    parser.add_argument('--verbose', dest='verbose', action='store_true',
+                        default=False,
+                        help='dump stdout,stderr to console. "debug" implies "verbose". "silent" implies not "verbose".')   
+
+    parser.add_argument('--ext', dest='sshscriptExt', action='store',
+                        default='.spy',
+                        help='the extension of sshscript file. default is .spy')
+
+    parser.add_argument('--debug', dest='debug', action='store_true',
+                        default=False,
                         help='set log level to debug')
 
-    # multiple values (requires no flag)
-    # path or folder, ex. python3 sshscript unitest
     parser.add_argument(dest='paths', action='store', nargs='*',
-                        help='the script file or folder of .spy files')
+                        help='path of .spy files or folders')
 
-    #args = parser.parse_args()
     args, unknown = parser.parse_known_args()
+    __main__.args = unknown
+
+    os.environ['SSHSCRIPT_EXT'] = args.sshscriptExt
+
     if len(args.paths):
-        main('-f',args)
+        main(args)
+
     else:
+        try:
+            from __init__ import __version__
+        except ImportError:
+            try:
+                from . import __version__
+            except ImportError:
+                __version__ = 'unknown'
+        print(f'SSHScript Version:{__version__}')
         parser.print_help()
 
+# SSHScriptDollar need this value to work 
+# when sshscript is imported as a module (main() not been called)
+__main__.SSHScript = SSHScript
+
+setupLogger()
+
 if __name__ == '__main__':
     run()
\ No newline at end of file
