@@ -1,5 +1,6 @@
+import __main__
 import time,subprocess
-import threading, os, sys
+import threading, os, sys, re
 from select import select
 import errno
 import asyncio
@@ -15,17 +16,23 @@ class GenericChannel(object):
         self.lock = threading.Lock()
         self._stdout = ''
         self._stderr = ''        
-        self.dump2sys = os.environ.get('VERBOSE',sys.stdout.isatty())
-        self.stdoutPrefix = os.environ.get('VERBOSE_STDOUT_PREFIX','▏').encode('utf8')
-        self.stderrPrefix = os.environ.get('VERBOSE_STDERR_PREFIX','🐞').encode('utf8')
-        self.stdoutDumpBuf = []
-        self.stderrDumpBuf = []
         self.stdoutBuf = []
         self.stderrBuf = []
         self.lock = threading.Lock()
         # 不要0，這樣可以讓一開始就先等一等
         self._lastIOTime = time.time()
 
+        # dump-related
+        if os.environ.get('VERBOSE') or os.environ.get('DEBUG'):
+            self.dump2sys = True
+            self.stdoutPrefix = os.environ.get('VERBOSE_STDOUT_PREFIX','▏').encode('utf8')
+            self.stderrPrefix = os.environ.get('VERBOSE_STDERR_PREFIX','🐞').encode('utf8')
+            self.stdoutDumpBuf = []
+            self.stderrDumpBuf = []
+        else:
+            #self.dump2sys = os.environ.get('VERBOSE',sys.stdout.isatty())
+            self.dump2sys = False
+
     @property
     def stdout(self):
         return self._stdout
@@ -69,7 +76,7 @@ class GenericChannel(object):
         sys.stdout.flush()
         sys.stderr.flush()
 
-    def expect(self,s,timeout=60,stdout=True,stderr=True):
+    def expect(self,pat,timeout=60,stdout=True,stderr=True):
         # this is a blocking function
         # 只會搜尋目前還buffred的資料，如果要搜尋新收到的資料，
         # 須在執行命令(seld.sendline())前先呼叫 self.recv()
@@ -79,15 +86,21 @@ class GenericChannel(object):
             targets.append(0)
         if stderr:
             targets.append(1)
+        if isinstance(pat,bytes):
+            pat = re.compile(pat.decode('utf8'),re.I)
+        elif isinstance(pat,str):
+            pat = re.compile(pat,re.I)
+        elif isinstance(pat,re.Pattern):
+            pass
+        else:
+            raise ValueError('expect() only accept str or re.Pattern')
         async def _wait():
-            startIdx = 0
             startTime = time.time()
-            b = s.encode('utf8')
             while True:
                 if time.time() - startTime > timeout:
-                    raise TimeoutError(f'Not found: {s} ' + '\n')
+                    raise TimeoutError(f'Not found: {pat} ' + '\n')
                 for i in targets:
-                    if data[i]().find(s) >= 0:
+                    if pat.search(data[i]()):
                         await self.waitio(1)
                         return
                 await asyncio.sleep(0.1)
@@ -106,16 +119,18 @@ class GenericChannel(object):
 
     def __enter__(self):
         return self
+    
     def __exit__(self,*args):
         self.wait(1)
         self.close()
     
     def sendline(self,s,secondsToWaitResponse=1):
+        __main__.SSHScript.logger.debug(f'sendline: {s}')
         if not s[-1] == '\n': s += '\n'
         # 確保跟前面的一個指令有點「距離」，不要在還在接收資料時送出下一個指令
         self.wait()
         n = self.send(s)
-        if secondsToWaitResponse is not None:
+        if secondsToWaitResponse > 0:
             self.recv(secondsToWaitResponse)
         return n
 
