@@ -1,7 +1,8 @@
 #!/usr/bin/env python3
-import os, sys, traceback, time, re, random, glob
-import queue, threading
-import subprocess, shlex, socket
+from glob import glob
+import os, re
+import subprocess, shlex
+import __main__
 import shutil, asyncio
 try:
     from sshscripterror import SSHScriptError
@@ -15,17 +16,9 @@ except ImportError:
     # pty is not available on Windows
     pty = None
 
-# replace @{var} in $shell-command
-#pvar = re.compile('(\b?)\@\{(.+)\}')
-# replace $.stdout, $.stderr to _c.stdout, _c.stderr, $.host in $LINE
-#pstd = re.compile('(\b?)\$(\.[a-z]+)\b?')
-# replace @{var} in py scripts
 pvarS = re.compile('(\b?)\@\{(.+?)\}',re.S)
 
-import __main__
-
-
-global logger
+global logger, loop
 logger = None
 loop = asyncio.get_event_loop()
 
@@ -34,8 +27,8 @@ class SSHScriptDollar(object):
     # aka $shell-commmand , or coverted "_c"
     def __init__(self,ssId,cmd=None,globals=None,locals=None,inWith=False):
         global logger
-        # assign logger 
-        if logger is None: logger = __main__.logger
+        # assign logger to sshscript's logger
+        if logger is None: logger = __main__.SSHScript.logger
         self.args = (ssId,cmd,globals,locals)
         self.sshscript = None #執行的脈絡下的 sshscript instance
         self.inWith = inWith
@@ -46,6 +39,7 @@ class SSHScriptDollar(object):
         self.bufferedErrorData = b''
         # set os.environ['NO_PTY']='1' to disable pty
         self.usePty = pty and os.environ.get('NO_PTY','') != '1'
+    
     def __call__(self,invokeShell=False,deepCall=True):
         ssId = self.args[0]
         self.sshscript = __main__.SSHScript.items[ssId] if ssId else __main__.SSHScript.inContext
@@ -73,32 +67,25 @@ class SSHScriptDollar(object):
             logger.debug(f'calling eval({m.group(2)}) in "${cmd}"')
             return f'{eval(m.group(2),_globals,_locals)}'
         cmd = pvarS.sub(pvarRepl,cmd)
-        return cmd
-
-    def execBySubprocess(self,invokeShell):
-        (ssId,cmd,_globals,_locals) = self.args
-        cmd = self.evalCommand()
         # should be one-line command
         cmds = [x.strip() for x in cmd.split('\n')]
         cmds = [x for x in cmds if (x and not x.startswith('#'))]
-        hasMultipleLines = len(cmds) > 1
+        return cmds
 
-        # prepare pre-assigned stdin
-        """
-        if self.cachedStdin:
-            if hasattr(self.cachedStdin,'read'):
-                input = self.cachedStdin.read()
-            else:
-                input = self.cachedStdin
-        else:
-            input = b''
-        """
+    def execBySubprocess(self,invokeShell):
+        global loop
+        #(ssId,cmd,_globals,_locals) = self.args
+        cmds = self.evalCommand()
+        # should be one-line command
+        #cmds = [x.strip() for x in cmd.split('\n')]
+        #cmds = [x for x in cmds if (x and not x.startswith('#'))]
+        #hasMultipleLines = len(cmds) > 1
 
         # 只要有with,一定是invokeShell(with ${} same as with $${})
         if self.inWith: invokeShell = True
 
         # implement stdin, and timeout (default to 60)
-        timeout = self.sshscript._timeout or 60    
+        timeout = float(os.environ.get('CMD_TIMEOUT',60))
         
         if invokeShell:
             self.invokeShell = True
@@ -109,10 +96,11 @@ class SSHScriptDollar(object):
                 if shCmd is None:
                     raise RuntimeError('no shell command found')
                         
-            if self.inWith or hasMultipleLines:
+            #if self.inWith or hasMultipleLines:
+            if 1:
                 # prepare popen command
                 args = shlex.split(shCmd)
-
+                logger.debug(f'subprocess.Popen {args}')
                 if self.usePty:
                     # ref: https://errorsfixing.com/run-interactive-bash-in-dumb-terminal-using-python-subprocess-popen-and-pty/
                     # ref: https://stackoverflow.com/questions/19880190/interactive-input-output-using-python
@@ -149,13 +137,15 @@ class SSHScriptDollar(object):
                     self.channel.wait(0.5)
                 except TimeoutError:
                     pass
-                for cmd in cmds:
-                    #print('send>>',cmd)
-                    self.channel.sendline(cmd)#(masterFd,(cmd+'\n').encode('utf8'))
-                    loop.run_until_complete(self.channel.waitio())
+                
+                for command in cmds:
+                    self.channel.sendline(command)
+                    #loop.run_until_complete(self.channel.waitio())
+                    self.channel.wait()
+                
                 if not self.inWith:
                     self.channel.close()
-
+            """
             else:
                 # inVokeShell, but only single line and not in with
                 # prepare popen command
@@ -181,6 +171,7 @@ class SSHScriptDollar(object):
                 self.channel.addStdoutData(outs)
                 self.channel.addStderrData(errs)                
                 self.channel.close()
+            """
         else:
             
             # not invoke shell
@@ -195,7 +186,7 @@ class SSHScriptDollar(object):
                 args = shlex.split(command)
                 if  ('|' in args) : raise SSHScriptError('| works only in shell mode, consider to run with $$',501)
                 if  ('>' in args) : raise SSHScriptError('> not works in $',502)
-                logger.debug(f'subprocess.Popen: {args}')
+                logger.debug(f'subprocess.Popen {args}')
                 cp = subprocess.Popen(args,
                     # 會引起  RuntimeWarning: line buffering (buffering=1) isn't supported in binary mode,
                     #bufsize=1,
@@ -203,9 +194,6 @@ class SSHScriptDollar(object):
                     stdin=subprocess.PIPE,stderr=subprocess.PIPE,stdout=subprocess.PIPE,
                     shell=invokeShell)
 
-                # implement stdin, and timeout (default to 600)
-                timeout = self.sshscript._timeout or 600
-
                 # outs,errs are bytes
                 try:
                     outs, errs = cp.communicate(timeout=timeout)
@@ -213,70 +201,66 @@ class SSHScriptDollar(object):
                     cp.kill()
                     outs, errs = cp.communicate()
                 self.stdin = cp.stdin
+                #多行時，$.stdout, $.stderr是所有的總和
                 self.channel.addStdoutData(outs)
                 self.channel.addStderrData(errs)
-            #多行時，$.stdout, $.stderr是所有的總和
-            if self.inWith:
-                # 不invoke shell，卻inWith，好像沒意義
-                # __exit__ will call self.channel.close()
-                pass
-            else:
-                # self.channel.close() will not call  self.channel.recv()
-                self.channel.recv()
-                self.channel.close()
-                if self.channel.stderr and self.sshscript._paranoid:
-                    raise SSHScriptError(self.channel.stderr)
+
+            # self.channel.close() will not call  self.channel.recv()
+            # so we need to call it here
+            self.channel.recv()
+            self.channel.close()
+            if self.channel.stderr and self.sshscript._paranoid:
+                raise SSHScriptError(self.channel.stderr)
     
     def execBySSH(self,invokeShell,deepCall):
-        (ssId,cmd,_globals,_locals) = self.args
+        global loop
+
+        cmds = self.evalCommand()
+        #cmds = [x.strip() for x in cmd.split('\n')]
+        #cmds = [x for x in cmds if (x and not x.startswith('#'))]
 
         # provide value of $.username, $.port, $.host
         self.host = self.sshscript.host;
         self.username = self.sshscript.username;
         self.port = self.sshscript.port
         
-        cmd = self.evalCommand()
-
-        #hasMultipleLines = '\n' in cmd
-        
-        cmds = [x.strip() for x in cmd.split('\n')]
-        cmds = [x for x in cmds if (x and not x.startswith('#'))]
         # 只要有with,一定是invokeShell(with ${} same as with $${})
         if self.inWith: invokeShell = True
 
         # implement stdin, and timeout (default to 60)
-        timeout = self.sshscript._timeout or 60            
+        timeout = float(os.environ.get('CMD_TIMEOUT',60))             
 
         client = self.sshscript.client if deepCall else self.sshscript._client
+        
         if invokeShell:
             self.invokeShell = True
-            logger.debug(f'executing in shell: {cmd}')
-            # $${...}
             # REF: https://stackoverflow.com/questions/6203653/how-do-you-execute-multiple-commands-in-a-single-session-in-paramiko-python/6203877#6203877
             
             self.channel = ParamikoChannel(self,client, timeout)
+            
             for command in cmds:
                 self.channel.sendline(command)# write to queue
                 self.channel.wait()
             
-            loop.run_until_complete(self.channel.waitio(1))
+            self.channel.wait(1)
+            #loop.run_until_complete(self.channel.waitio(1))
             if not self.inWith:
                 self.channel.close()
 
         else:
-            # hasMultipleLines:
-            # ${....}
             self.channel = ParamikoChannel(self,None, timeout)
             for command in cmds:
-                logger.debug(f'executing {command}')
+                logger.debug(f'ssh executing {command}')
                 # The paramiko documentation says:
                 # "using exec_command or invoke_shell without a pty will ever have data on the stderr stream."
                 # So, we always need not a pty.
-                stdin, stdout,stderr = client.exec_command(command,get_pty=0,timeout=self.sshscript._timeout)
+                stdin, stdout,stderr = client.exec_command(command,get_pty=0,timeout=timeout)
                 self.stdin = stdin
                 self.channel.addStdoutData(stdout.read())
                 self.channel.addStderrData(stderr.read())
-            # self.channel.close() will not call  self.channel.recv()
+
+            # self.channel.close() will not call self.channel.recv()
+            # so, we need to call it here
             self.channel.recv()
             self.channel.close()
             if self.channel.stderr and self.sshscript._paranoid:
